@@ -24,6 +24,7 @@ such a Python configuration file.
 
 import sys
 import sysconfig
+import os
 import os.path
 import platform
 import ctypes
@@ -32,14 +33,28 @@ import logging
 import subprocess
 
 
-__author__ = "Norman Fomferra, Brockmann Consult GmbH"
-__copyright__ = "Copyright 2015-2017 Brockmann Consult GmbH"
+__author__ = "Norman Fomferra (Brockmann Consult GmbH) and contributors"
+__copyright__ = "Copyright 2015-2018 Brockmann Consult GmbH and contributors"
 __license__ = "Apache 2.0"
-__version__ = "0.9.0"
+__version__ = "0.13.0"
 
 
-# Uncomment for debugging
-# logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.DEBUG)
+# Setup a dedicated logger for jpyutil.
+# This way importing jpyutil does not interfere with logging in other modules
+logger = logging.getLogger('jpyutil')
+# Get log level from environment variable JPY_LOG_LEVEL. Default to INFO
+log_level = os.getenv('JPY_LOG_LEVEL', 'INFO')
+try:
+    logger.setLevel(getattr(logging, log_level))
+except AttributeError as ex:
+    print('JPY_LOG_LEVEL must be DEBUG, INFO, WARNING, ERROR or CRITICAL')
+    raise ex
+
+ch = logging.StreamHandler()
+ch.setLevel(getattr(logging, log_level))
+formatter = logging.Formatter('%(name)s - %(levelname)s: %(message)s')
+ch.setFormatter(formatter)
+logger.addHandler(ch)
 
 JDK_HOME_VARS = ('JPY_JAVA_HOME', 'JPY_JDK_HOME', 'JAVA_HOME', 'JDK_HOME',)
 JRE_HOME_VARS = ('JPY_JAVA_HOME', 'JPY_JDK_HOME', 'JPY_JRE_HOME', 'JAVA_HOME', 'JDK_HOME', 'JRE_HOME', 'JAVA_JRE')
@@ -69,41 +84,37 @@ def _get_unique_config_values(names):
     values = []
     for name in names:
         value = sysconfig.get_config_var(name)
-        if value and not value in values:
+        if value and value not in values:
             values.append(value)
     return values
 
 
 def _add_paths_if_exists(path_list, *paths):
     for path in paths:
-        if os.path.exists(path) and not path in path_list:
+        if os.path.exists(path) and path not in path_list:
             path_list.append(path)
     return path_list
 
+def _find_loadable_spec_origin(name):
+    import importlib.util
+    spec = importlib.util.find_spec(name)
+    return spec.origin if spec and spec.has_location else None
 
 def _get_module_path(name, fail=False, install_path=None):
     """ Find the path to the jpy jni modules. """
-    import imp
-
-    module = imp.find_module(name)
-    if not module and fail:
-        raise RuntimeError("can't find module '" + name + "'")
-    path = module[1]
-    if not path and fail:
-        raise RuntimeError("module '" + name + "' is missing a file path")
-    
-    if install_path:
-        return os.path.join(install_path, os.path.split(path)[1])
-
-    return path
-
+    spec_origin = _find_loadable_spec_origin(name)
+    if not spec_origin:
+        if fail:
+            raise ImportError("module '{}' not found, or is not loadable".format(name))
+        return None
+    return spec_origin if not install_path else os.path.join(install_path, os.path.split(spec_origin)[1])
 
 def _find_file(search_dirs, *filenames):
     for filename in filenames:
-        for dir in search_dirs:
-            path = os.path.normpath(os.path.join(dir, filename))
+        for dir_ in search_dirs:
+            path = os.path.normpath(os.path.join(dir_, filename))
             path_exists = os.path.exists(path)
-            logging.debug("Exists '%s'? %s" % (path, "yes" if path_exists else "no"))
+            logger.debug("Exists '%s'? %s" % (path, "yes" if path_exists else "no"))
             if path_exists:
                 return path
     return None
@@ -131,11 +142,11 @@ def find_jdk_home_dir():
                 and os.path.exists(os.path.join(jdk_home_dir, 'include')) \
                 and os.path.exists(os.path.join(jdk_home_dir, 'lib')):
             return jdk_home_dir
-    logging.debug('Checking Maven for JAVA_HOME...')
+    logger.debug('Checking Maven for JAVA_HOME...')
     try:
         output = subprocess.check_output(['mvn', '-v'])
         if isinstance(output, bytes) and not isinstance(output, str):
-            #PY3 related
+            # In Python 3, byte arrays must be explicitly converted into strings.
             output = output.decode('utf-8')
         for part in output.split('\n'):
             if part.startswith('Java home:'):
@@ -145,19 +156,31 @@ def find_jdk_home_dir():
                 
     except Exception:
         # maven probably isn't installed or not on PATH
-        logging.debug('Maven not found on PATH. No JAVA_HOME found.')
+        logger.debug('Maven not found on PATH. No JAVA_HOME found.')
 
     return None
 
+def _java_process_java_home():
+    logger.debug('Checking Java for JAVA_HOME...')
+    try:
+        from java_utilities import lookup_property
+        return lookup_property('java.home', use_env=False)
+    except ImportError:
+        logger.debug("java_utilities not found, skipping java process check")
+    except Exception as e:
+        logger.debug(e, exc_info=True)
+    return None
 
 def find_jvm_dll_file(java_home_dir=None, fail=False):
     """
     Try to detect the JVM's shared library file.
     :param java_home_dir: The Java JRE or JDK installation directory to be used for searching.
+    :param fail: Controls behaviour when library file cannot be found. If False, use a fallback. If True, raise
+                a runtime error.
     :return: pathname if found, else None
     """
 
-    logging.debug("Searching for JVM shared library file")
+    logger.debug("Searching for JVM shared library file")
 
     if java_home_dir:
         jvm_dll_path = _find_jvm_dll_file(java_home_dir)
@@ -175,9 +198,15 @@ def find_jvm_dll_file(java_home_dir=None, fail=False):
             if jvm_dll_path:
                 return jvm_dll_path
 
+    java_home_dir = _java_process_java_home()
+    if java_home_dir:
+        jvm_dll_path = _find_jvm_dll_file(java_home_dir)
+        if jvm_dll_path:
+            return jvm_dll_path
+
     jvm_dll_path = ctypes.util.find_library(JVM_LIB_NAME)
     if jvm_dll_path:
-        logging.debug("No JVM shared library file found in all search paths. Using fallback %s" % repr(jvm_dll_path))
+        logger.debug("No JVM shared library file found in all search paths. Using fallback %s" % repr(jvm_dll_path))
     elif fail:
         raise RuntimeError("can't find any JVM shared library")
 
@@ -203,15 +232,15 @@ def _get_jvm_lib_dirs(java_home_dir):
 
 def _get_existing_subdirs(dirs, subdirname):
     new_dirs = []
-    for dir in dirs:
-        new_dir = os.path.join(dir, subdirname)
+    for dir_ in dirs:
+        new_dir = os.path.join(dir_, subdirname)
         if os.path.isdir(new_dir):
             new_dirs.append(new_dir)
     return new_dirs
 
 
 def _find_jvm_dll_file(java_home_dir):
-    logging.debug("Searching for JVM shared library file in %s" % repr(java_home_dir))
+    logger.debug("Searching for JVM shared library file in %s" % repr(java_home_dir))
 
     if not os.path.exists(java_home_dir):
         return None
@@ -234,17 +263,15 @@ def _find_jvm_dll_file(java_home_dir):
 
 
 def _find_python_dll_file(fail=False):
-    logging.debug("Searching for Python shared library file")
+    logger.debug("Searching for Python shared library file")
 
-    #
     # Prepare list of search directories
-    #
 
     search_dirs = [sys.prefix]
 
     extra_search_dirs = [sysconfig.get_config_var(name) for name in PYTHON_LIB_DIR_CONFIG_VAR_NAMES]
     for extra_dir in extra_search_dirs:
-        if extra_dir and not extra_dir in search_dirs and os.path.exists(extra_dir):
+        if extra_dir and extra_dir not in search_dirs and os.path.exists(extra_dir):
             search_dirs.append(extra_dir)
 
     if platform.system() == 'Windows':
@@ -258,11 +285,9 @@ def _find_python_dll_file(fail=False):
         extra_search_dirs = _get_existing_subdirs(search_dirs, multi_arch_sub_dir)
         search_dirs = extra_search_dirs + search_dirs
 
-    logging.debug("Potential Python shared library search dirs: %s" % repr(search_dirs))
+    logger.debug("Potential Python shared library search dirs: %s" % repr(search_dirs))
 
-    #
     # Prepare list of possible library file names
-    #
 
     vmaj = str(sys.version_info.major)
     vmin = str(sys.version_info.minor)
@@ -278,7 +303,7 @@ def _find_python_dll_file(fail=False):
         versions = (vmaj + "." + vmin, vmaj, '')
         file_names = ['libpython' + v + '.so' for v in versions]
 
-    logging.debug("Potential Python shared library file names: %s" % repr(file_names))
+    logger.debug("Potential Python shared library file names: %s" % repr(file_names))
 
     python_dll_path = _find_file(search_dirs, *file_names)
     if python_dll_path:
@@ -286,7 +311,7 @@ def _find_python_dll_file(fail=False):
 
     python_dll_path = ctypes.util.find_library(PYTHON_LIB_NAME)
     if python_dll_path:
-        logging.debug(
+        logger.debug(
             "No Python shared library file found in all search paths. Using fallback %s" % repr(python_dll_path))
     elif fail:
         raise RuntimeError("can't find any Python shared library")
@@ -308,7 +333,6 @@ def _get_python_api_config(config_file=None):
     try:
         # 2. Try Python import machinery
         import jpyconfig
-
         return jpyconfig
 
     except ImportError:
@@ -340,10 +364,10 @@ def preload_jvm_dll(jvm_dll_file=None,
         jvm_dll_file = find_jvm_dll_file(java_home_dir=java_home_dir, fail=fail)
 
     if jvm_dll_file:
-        logging.debug('Preloading JVM shared library %s' % repr(jvm_dll_file))
+        logger.debug('Preloading JVM shared library %s' % repr(jvm_dll_file))
         return ctypes.CDLL(jvm_dll_file, mode=ctypes.RTLD_GLOBAL)
     else:
-        logging.warning('Failed to preload JVM shared library. No shared library found.')
+        logger.warning('Failed to preload JVM shared library. No shared library found.')
         return None
 
 
@@ -434,8 +458,19 @@ def init_jvm(java_home=None,
                                       jvm_properties=jvm_properties,
                                       jvm_options=jvm_options,
                                       config=config)
-        logging.debug('Creating JVM with options %s' % repr(jvm_options))
+        logger.debug('Creating JVM with options %s' % repr(jvm_options))
         jpy.create_jvm(options=jvm_options)
+        try:
+            py_lib_initializer = jpy.get_type('org.jpy.PyLibInitializer')
+            py_lib_initializer.initPyLib(
+                _find_python_dll_file(fail=True),
+                _get_module_path('jpy', fail=True),
+                _get_module_path('jdl', fail=True)
+            )
+        except ValueError:
+            # It's valid to not have jpy.jar on the classpath if you don't expect java to call into python
+            logger.debug("Unable to find org.jpy.PyLibInitializer on classpath")
+            pass
     else:
         jvm_options = None
 
@@ -468,7 +503,7 @@ class Properties:
 
     def set_property(self, key, value):
         if value:
-            if not key in self.keys:
+            if key not in self.keys:
                 self.keys.append(key)
             self.values[key] = value
         else:
@@ -519,7 +554,9 @@ def write_config_files(out_dir='.',
                        jvm_dll_file=None,
                        install_dir=None,
                        req_java_api_conf=True,
-                       req_py_api_conf=True):
+                       req_py_api_conf=True,
+                       jvm_properties=None,
+                       jvm_options=None):
     """
     Writes the jpy configuration files for Java and/or Python.
 
@@ -554,15 +591,21 @@ def write_config_files(out_dir='.',
                 f.write('jvm_dll = %s\n' % repr(jvm_dll_file))
                 f.write('jvm_maxmem = None\n')
                 f.write('jvm_classpath = []\n')
-                f.write('jvm_properties = {}\n')
-                f.write('jvm_options = []\n')
-            logging.info("jpy Python API configuration written to '%s'" % py_api_config_file)
+                if jvm_properties is None:
+                    f.write('jvm_properties = {}\n')
+                else:
+                    f.write('jvm_properties = ' + str(jvm_properties) + '\n')
+                if jvm_options is None:
+                    f.write('jvm_options = []\n')
+                else:
+                    f.write('jvm_options = [\'' + jvm_options + '\']\n')
+            logger.info("jpy Python API configuration written to '%s'" % py_api_config_file)
         except Exception:
-            logging.exception("Error while writing Python API configuration")
+            logger.exception("Error while writing Python API configuration")
             if req_py_api_conf:
                 retcode = 1
     else:
-        logging.error("Can't determine any JVM shared library")
+        logger.error("Can't determine any JVM shared library")
         if req_py_api_conf:
             retcode = 2
 
@@ -572,9 +615,9 @@ def write_config_files(out_dir='.',
         java_api_properties.store(java_api_config_file, comments=[
             "Created by '%s' tool on %s" % (tool_name, str(datetime.datetime.now())),
             "This file is read by the jpy Java API (org.jpy.PyLib class) in order to find shared libraries"])
-        logging.info("jpy Java API configuration written to '%s'" % java_api_config_file)
+        logger.info("jpy Java API configuration written to '%s'" % java_api_config_file)
     except Exception:
-        logging.exception("Error while writing Java API configuration")
+        logger.exception("Error while writing Java API configuration")
         if req_java_api_conf:
             retcode = 3
 
@@ -606,16 +649,17 @@ def _main():
     parser.add_argument("--install_dir", action='store', default=None, help="Optional. Used during pip install of JPY")
     args = parser.parse_args()
 
-    log_level = getattr(logging, args.log_level.upper(), None)
-    if not isinstance(log_level, int):
-        raise ValueError('Invalid log level: %s' % log_level)
+    # Using _ suffix to avoid shadowing global log_level variable
+    log_level_ = getattr(logging, args.log_level.upper(), None)
+    if not isinstance(log_level_, int):
+        raise ValueError('Invalid log level: %s' % log_level_)
 
     log_format = '%(levelname)s: %(message)s'
     log_file = args.log_file
     if log_file:
-        logging.basicConfig(format=log_format, level=log_level, filename=log_file, filemode='w')
+        logging.basicConfig(format=log_format, level=log_level_, filename=log_file, filemode='w')
     else:
-        logging.basicConfig(format=log_format, level=log_level)
+        logging.basicConfig(format=log_format, level=log_level_)
 
     try:
         retcode = write_config_files(out_dir=args.out,
@@ -625,11 +669,11 @@ def _main():
                                      req_py_api_conf=args.req_py,
                                      install_dir=args.install_dir)
     except:
-        logging.exception("Configuration failed")
+        logger.exception("Configuration failed")
         retcode = 100
 
     if retcode == 0:
-        logging.info("Configuration completed successfully")
+        logger.info("Configuration completed successfully")
 
     exit(retcode)
 
