@@ -12,6 +12,9 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
+ *
+ * This file was modified by Deephaven Data Labs.
+ *
  */
 
 #include "jpy_module.h"
@@ -23,26 +26,24 @@
 #include "jpy_jfield.h"
 #include "jpy_conv.h"
 
-JPy_JObj* JObj_New(JNIEnv* jenv, jobject objectRef)
+PyObject* JObj_New(JNIEnv* jenv, jobject objectRef)
 {
-    jclass classRef;
     JPy_JType* type;
-
-    classRef = (*jenv)->GetObjectClass(jenv, objectRef);
-    type = JType_GetType(jenv, classRef, JNI_TRUE);
-    (*jenv)->DeleteLocalRef(jenv, classRef);
+    type = JType_GetTypeForObject(jenv, objectRef, JNI_TRUE);
     if (type == NULL) {
         return NULL;
     }
-
     return JObj_FromType(jenv, type, objectRef);
 }
 
-JPy_JObj* JObj_FromType(JNIEnv* jenv, JPy_JType* type, jobject objectRef)
+PyObject* JObj_FromType(JNIEnv* jenv, JPy_JType* type, jobject objectRef)
 {
+    PyObject* callable;
+    PyObject* callableResult;
+
     JPy_JObj* obj;
 
-    obj = (JPy_JObj*) PyObject_New(JPy_JObj, (PyTypeObject*) type);
+    obj = (JPy_JObj*) PyObject_New(JPy_JObj, JTYPE_AS_PYTYPE(type));
     if (obj == NULL) {
         return NULL;
     }
@@ -61,26 +62,37 @@ JPy_JObj* JObj_FromType(JNIEnv* jenv, JPy_JType* type, jobject objectRef)
 
         array = (JPy_JArray*) obj;
         array->bufferExportCount = 0;
+        array->buf = NULL;
     }
 
-    return obj;
+    // we check the type translations dictionary for a callable for this java type name,
+    // and apply the returned callable to the wrapped object
+    callable = PyDict_GetItemString(JPy_Type_Translations, type->javaName);
+    if (callable != NULL) {
+        if (PyCallable_Check(callable)) {
+            callableResult = PyObject_CallFunction(callable, "OO", type, obj);
+            if (callableResult == NULL) {
+                return Py_None;
+            } else {
+                return callableResult;
+            }
+        }
+    }
+
+    return (PyObject *)obj;
 }
 
-/**
- * The JObj type's tp_init slot. Called when the type is used to create new instances (constructor).
- */
-int JObj_init(JPy_JObj* self, PyObject* args, PyObject* kwds)
+int JObj_init_internal(JNIEnv* jenv, JPy_JObj* self, PyObject* args, PyObject* kwds)
 {
-    JNIEnv* jenv;
     PyTypeObject* type;
     JPy_JType* jType;
     PyObject* constructor;
     JPy_JMethod* jMethod;
-    jobject objectRef;
+    jobject localObjectRef;
+    jobject globalObjectRef;
     jvalue* jArgs;
     JPy_ArgDisposer* jDisposers;
-
-    JPy_GET_JNI_ENV_OR_RETURN(jenv, -1)
+    int isVarArgsArray;
 
     type = ((PyObject*) self)->ob_type;
 
@@ -101,21 +113,21 @@ int JObj_init(JPy_JObj* self, PyObject* args, PyObject* kwds)
         return -1;
     }
 
-    jMethod = JOverloadedMethod_FindMethod(jenv, (JPy_JOverloadedMethod*) constructor, args, JNI_FALSE);
+    jMethod = JOverloadedMethod_FindMethod(jenv, (JPy_JOverloadedMethod*) constructor, args, JNI_FALSE, &isVarArgsArray);
     if (jMethod == NULL) {
         return -1;
     }
 
-    if (JMethod_CreateJArgs(jenv, jMethod, args, &jArgs, &jDisposers) < 0) {
+    if (JMethod_CreateJArgs(jenv, jMethod, args, &jArgs, &jDisposers, isVarArgsArray) < 0) {
         return -1;
     }
 
     JPy_DIAG_PRINT(JPy_DIAG_F_MEM, "JObj_init: calling Java constructor %s\n", jType->javaName);
 
-    objectRef = (*jenv)->NewObjectA(jenv, jType->classRef, jMethod->mid, jArgs);
+    localObjectRef = (*jenv)->NewObjectA(jenv, jType->classRef, jMethod->mid, jArgs);
     JPy_ON_JAVA_EXCEPTION_RETURN(-1);
 
-    if (objectRef == NULL) {
+    if (localObjectRef == NULL) {
         PyErr_NoMemory();
         return -1;
     }
@@ -124,33 +136,54 @@ int JObj_init(JPy_JObj* self, PyObject* args, PyObject* kwds)
         JMethod_DisposeJArgs(jenv, jMethod->paramCount, jArgs, jDisposers);
     }
 
-    objectRef = (*jenv)->NewGlobalRef(jenv, objectRef);
-    if (objectRef == NULL) {
+    globalObjectRef = (*jenv)->NewGlobalRef(jenv, localObjectRef);
+    if (globalObjectRef == NULL) {
         PyErr_NoMemory();
         return -1;
     }
+    JPy_DELETE_LOCAL_REF(localObjectRef);
 
     // Note:  __init__ may be called multiple times, so we have to release the old objectRef
     if (self->objectRef != NULL) {
         (*jenv)->DeleteGlobalRef(jenv, self->objectRef);
     }
 
-    self->objectRef = objectRef;
+    self->objectRef = globalObjectRef;
 
     JPy_DIAG_PRINT(JPy_DIAG_F_MEM, "JObj_init: self->objectRef=%p\n", self->objectRef);
 
     return 0;
 }
 
+/**
+ * The JObj type's tp_init slot. Called when the type is used to create new instances (constructor).
+ */
+int JObj_init(JPy_JObj* self, PyObject* args, PyObject* kwds)
+{
+    JPy_FRAME(int, -1, JObj_init_internal(jenv, self, args, kwds), 16)
+}
+
 /**
  * The JObj type's tp_dealloc slot. Called when the reference count reaches zero.
  */
 void JObj_dealloc(JPy_JObj* self)
 {
     JNIEnv* jenv;
+    JPy_JType* jtype;
 
     JPy_DIAG_PRINT(JPy_DIAG_F_MEM, "JObj_dealloc: releasing instance of %s, self->objectRef=%p\n", Py_TYPE(self)->tp_name, self->objectRef);
 
+    jtype = (JPy_JType *)Py_TYPE(self);
+    if (jtype->componentType != NULL && jtype->componentType->isPrimitive) {
+        JPy_JArray* array;
+        array = (JPy_JArray*) self;
+        assert(array->bufferExportCount == 0);
+        if (array->buf != NULL) {
+            JArray_ReleaseJavaArrayElements(array, array->javaType);
+        }
+  
+    }    
+
     jenv = JPy_GetJNIEnv();
     if (jenv != NULL) {
         if (self->objectRef != NULL) {
@@ -191,10 +224,10 @@ int JObj_Equals(JNIEnv* jenv, JPy_JObj* obj1, JPy_JObj* obj2)
     ref1 = obj1->objectRef;
     ref2 = obj2->objectRef;
 
-    if ((*jenv)->IsSameObject(jenv, ref1, ref2)) {
+    if ((*jenv)->IsSameObject(jenv, ref1, ref2) || (*jenv)->CallBooleanMethod(jenv, ref1, JPy_Object_Equals_MID, ref2)) {
         returnValue = 1;
     } else {
-        returnValue = (*jenv)->CallIntMethod(jenv, ref1, JPy_Object_Equals_MID, ref2);
+        returnValue = 0;
     }
     (*jenv)->ExceptionClear(jenv); // we can't deal with exceptions here, so clear any
     return returnValue;
@@ -318,7 +351,7 @@ PyObject* JObj_str(JPy_JObj* self)
     returnValue = JPy_FromJString(jenv, stringRef);
 
 error:
-    (*jenv)->DeleteLocalRef(jenv, stringRef);
+    JPy_DELETE_LOCAL_REF(stringRef);
 
     return returnValue;
 }
@@ -478,7 +511,7 @@ PyObject* JObj_getattro(JPy_JObj* self, PyObject* name)
             jobject item = (*jenv)->GetObjectField(jenv, self->objectRef, field->fid);
             JPy_ON_JAVA_EXCEPTION_RETURN(NULL);
             returnValue = JPy_FromJObjectWithType(jenv, item, field->type);
-            (*jenv)->DeleteLocalRef(jenv, item);
+            JPy_DELETE_LOCAL_REF(item);
             return returnValue;
         }
     } else {
@@ -577,7 +610,7 @@ PyObject* JObj_sq_item(JPy_JObj* self, Py_ssize_t index)
         jobject item = (*jenv)->GetObjectArrayElement(jenv, self->objectRef, (jsize) index);
         JPy_ON_JAVA_EXCEPTION_RETURN(NULL);
         returnValue = JPy_FromJObjectWithType(jenv, item, type->componentType);
-        (*jenv)->DeleteLocalRef(jenv, item);
+        JPy_DELETE_LOCAL_REF(item);
         return returnValue;
     }
 }
@@ -678,21 +711,25 @@ int JType_InitSlots(JPy_JType* type)
     isArray = type->componentType != NULL;
     isPrimitiveArray = isArray && type->componentType->isPrimitive;
 
-    typeObj = (PyTypeObject*) type;
+    typeObj = JTYPE_AS_PYTYPE(type);
 
-    Py_REFCNT(typeObj) = 1;
-    Py_TYPE(typeObj) = NULL;
-    Py_SIZE(typeObj) = 0;
+    #if defined(JPY_COMPAT_39P)
+        Py_SET_REFCNT(typeObj, 1);
+    #else
+        Py_REFCNT(typeObj) = 1;
+    #endif
+    Py_SET_TYPE(typeObj, NULL);
+    Py_SET_SIZE(typeObj, 0);
     // todo: The following lines are actually correct, but setting Py_TYPE(type) = &JType_Type results in an interpreter crash. Why?
     // This is still a problem because all the JType slots are actually never called (especially JType_getattro is
     // needed to resolve unresolved JTypes and to recognize static field and methods access)
-    //Py_INCREF(&JType_Type);
-    //Py_TYPE(type) = &JType_Type;
-    //Py_SIZE(type) = sizeof (JPy_JType);
+    //JPy_INCREF(&JType_Type);
+    //Py_SET_TYPE(type, &JType_Type);
+    //Py_SET_SIZE(type, sizeof (JPy_JType));
 
     typeObj->tp_basicsize = isPrimitiveArray ? sizeof (JPy_JArray) : sizeof (JPy_JObj);
     typeObj->tp_itemsize = 0;
-    typeObj->tp_base = type->superType != NULL ? (PyTypeObject*) type->superType : &JType_Type;
+    typeObj->tp_base = type->superType != NULL ? JTYPE_AS_PYTYPE(type->superType) : &JType_Type;
     //typeObj->tp_base = (PyTypeObject*) type->superType;
     typeObj->tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE;
     // If I uncomment the following line, I get (unpredictable) interpreter crashes
@@ -772,7 +809,7 @@ int JType_InitSlots(JPy_JType* type)
 }
 
 // This is only a good test as long JObj_init() is not used in other types
-#define JPY_IS_JTYPE(T)  (((PyTypeObject*) T)->tp_init == (initproc) JObj_init)
+#define JPY_IS_JTYPE(T)  (JTYPE_AS_PYTYPE(T)->tp_init == (initproc) JObj_init)
 
 
 int JObj_Check(PyObject* arg)
