@@ -12,6 +12,9 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
+ *
+ * This file was modified by Deephaven Data Labs.
+ *
  */
 
 #ifndef JPY_JOBJ_H
@@ -37,8 +40,8 @@ JPy_JObj;
 
 int JObj_Check(PyObject* arg);
 
-JPy_JObj* JObj_New(JNIEnv* jenv, jobject objectRef);
-JPy_JObj* JObj_FromType(JNIEnv* jenv, JPy_JType* type, jobject objectRef);
+PyObject* JObj_New(JNIEnv* jenv, jobject objectRef);
+PyObject* JObj_FromType(JNIEnv* jenv, JPy_JType* type, jobject objectRef);
 
 int JObj_InitTypeSlots(PyTypeObject* type, const char* typeName, PyTypeObject* superType);
 
